@@ -39,15 +39,20 @@ typedef enum {
 	E_BOOK_ERROR_AUTHENTICATION_FAILED,
 	E_BOOK_ERROR_AUTHENTICATION_REQUIRED,
 	E_BOOK_ERROR_TLS_NOT_AVAILABLE,
-	E_BOOK_ERROR_CORBA_EXCEPTION,
+	E_BOOK_ERROR_DBUS_EXCEPTION,
 	E_BOOK_ERROR_NO_SUCH_SOURCE,
 	E_BOOK_ERROR_OFFLINE_UNAVAILABLE,
 	E_BOOK_ERROR_OTHER_ERROR,
 	E_BOOK_ERROR_INVALID_SERVER_VERSION,
 	E_BOOK_ERROR_UNSUPPORTED_AUTHENTICATION_METHOD,
-	E_BOOK_ERROR_NO_SPACE
+	E_BOOK_ERROR_NO_SPACE,
+	E_BOOK_ERROR_NOT_SUPPORTED
 } EBookStatus;
 
+#ifndef E_BOOK_DISABLE_DEPRECATED
+#define E_BOOK_ERROR_CORBA_EXCEPTION E_BOOK_ERROR_DBUS_EXCEPTION
+#endif
+
 typedef enum {
 	E_BOOK_VIEW_STATUS_OK,
 	E_BOOK_VIEW_STATUS_TIME_LIMIT_EXCEEDED,
