@@ -19,8 +19,12 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef _CAMEL_CHARSET_MAP_H
-#define _CAMEL_CHARSET_MAP_H
+#if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
+#error "Only <camel/camel.h> can be included directly."
+#endif
+
+#ifndef CAMEL_CHARSET_MAP_H
+#define CAMEL_CHARSET_MAP_H
 
 #include <glib.h>
 
@@ -45,4 +49,4 @@ const gchar *camel_charset_iso_to_windows (const gchar *isocharset);
 
 G_END_DECLS
 
-#endif /* _CAMEL_CHARSET_MAP_H */
+#endif /* CAMEL_CHARSET_MAP_H */
