@@ -19,13 +19,10 @@
  * USA
  */
 
-#include <pthread.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <libedataserver/e-data-server-util.h>
-
 #include "camel-debug.h"
 
 gint camel_verbose_debug;
@@ -36,10 +33,7 @@ static GHashTable *debug_table = NULL;
  * camel_debug_init:
  * @void:
  *
- * Init camel debug.  Maintain legacy CAMEL_VERBOSE_DEBUG as well as the
- * new CAMEL_DEBUG based environment variable interfaces.
- *
- * CAMEL_VERBOSE_DEBUG is set to a number to turn debug on.
+ * Init camel debug.
  *
  * CAMEL_DEBUG is set to a comma separated list of modules to debug.
  * The modules can contain module-specific specifiers after a ':', or
@@ -47,14 +41,11 @@ static GHashTable *debug_table = NULL;
  * for imap debug, or 'imap:folder' for imap folder debug.  Additionaly,
  * ':folder' can be used for a wildcard for any folder operations.
  **/
-void camel_debug_init(void)
+void
+camel_debug_init (void)
 {
 	gchar *d;
 
-	d = getenv("CAMEL_VERBOSE_DEBUG");
-	if (d)
-		camel_verbose_debug = atoi(d);
-
 	d = g_strdup(getenv("CAMEL_DEBUG"));
 	if (d) {
 		gchar *p;
@@ -118,7 +109,7 @@ gboolean camel_debug(const gchar *mode)
 	return FALSE;
 }
 
-static pthread_mutex_t debug_lock = PTHREAD_MUTEX_INITIALIZER;
+static GStaticMutex debug_lock = G_STATIC_MUTEX_INIT;
 /**
  * camel_debug_start:
  * @mode:
@@ -133,8 +124,8 @@ gboolean
 camel_debug_start(const gchar *mode)
 {
 	if (camel_debug(mode)) {
-		pthread_mutex_lock(&debug_lock);
-		printf("Thread %" G_GINT64_MODIFIER "x >\n", e_util_pthread_id(pthread_self()));
+		g_static_mutex_lock (&debug_lock);
+		printf ("Thread %p >\n", g_thread_self());
 		return TRUE;
 	}
 
@@ -150,8 +141,8 @@ camel_debug_start(const gchar *mode)
 void
 camel_debug_end(void)
 {
-	printf("< %" G_GINT64_MODIFIER "x >\n", e_util_pthread_id(pthread_self()));
-	pthread_mutex_unlock(&debug_lock);
+	printf ("< %p >\n", g_thread_self());
+	g_static_mutex_unlock (&debug_lock);
 }
 
 #if 0
