@@ -619,9 +619,9 @@ do_label (struct _ESExp *f, gint argc, struct _ESExpResult **argv, CamelFilterDr
 
 		label = argv[0]->value.string;
 
-		for (i = 0; new_labels [i]; i++) {
-			if (label && strcmp (new_labels [i] + 6, label) == 0) {
-				label = new_labels [i];
+		for (i = 0; new_labels[i]; i++) {
+			if (label && strcmp (new_labels[i] + 6, label) == 0) {
+				label = new_labels[i];
 				break;
 			}
 		}
@@ -1230,7 +1230,7 @@ camel_filter_driver_filter_mbox (CamelFilterDriver *driver,
 	gint i = 0;
 	struct stat st;
 	gint status;
-	off_t last = 0;
+	goffset last = 0;
 	gint ret = -1;
 
 	fd = g_open (mbox, O_RDONLY|O_BINARY, 0);
