@@ -22,8 +22,12 @@
  * USA
  */
 
+#if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
+#error "Only <camel/camel.h> can be included directly."
+#endif
+
 #ifndef CAMEL_UID_CACHE_H
-#define CAMEL_UID_CACHE_H 1
+#define CAMEL_UID_CACHE_H
 
 #include <glib.h>
 
