@@ -30,16 +30,9 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include <glib.h>
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
-#include "camel-db.h"
-#include "camel-file-utils.h"
-#include "camel-mime-message.h"
-#include "camel-stream-null.h"
-#include "camel-string-utils.h"
-
 #include "camel-local-summary.h"
 
 #define w(x)
@@ -50,7 +43,7 @@
 
 #define EXTRACT_FIRST_DIGIT(val) val=strtoul (part, &part, 10);
 
-static CamelFIRecord * summary_header_to_db (CamelFolderSummary *, CamelException *ex);
+static CamelFIRecord * summary_header_to_db (CamelFolderSummary *, GError **error);
 static gint summary_header_from_db (CamelFolderSummary *, CamelFIRecord *);
 
 static gint summary_header_load (CamelFolderSummary *, FILE *);
@@ -61,81 +54,80 @@ static CamelMessageInfo * message_info_new_from_header (CamelFolderSummary *, st
 static gint local_summary_decode_x_evolution(CamelLocalSummary *cls, const gchar *xev, CamelLocalMessageInfo *mi);
 static gchar *local_summary_encode_x_evolution(CamelLocalSummary *cls, const CamelLocalMessageInfo *mi);
 
-static gint local_summary_load(CamelLocalSummary *cls, gint forceindex, CamelException *ex);
-static gint local_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, CamelException *ex);
-static gint local_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changeinfo, CamelException *ex);
-static CamelMessageInfo *local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMessageInfo *info, CamelFolderChangeInfo *, CamelException *ex);
+static gint local_summary_load(CamelLocalSummary *cls, gint forceindex, GError **error);
+static gint local_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, GError **error);
+static gint local_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changeinfo, GError **error);
+static CamelMessageInfo *local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMessageInfo *info, CamelFolderChangeInfo *, GError **error);
 static gint local_summary_need_index(void);
 
-static void camel_local_summary_class_init (CamelLocalSummaryClass *klass);
-static void camel_local_summary_init       (CamelLocalSummary *obj);
-static void camel_local_summary_finalise   (CamelObject *obj);
-static CamelFolderSummaryClass *camel_local_summary_parent;
+G_DEFINE_TYPE (CamelLocalSummary, camel_local_summary, CAMEL_TYPE_FOLDER_SUMMARY)
 
-CamelType
-camel_local_summary_get_type(void)
+static void
+local_summary_dispose (GObject *object)
 {
-	static CamelType type = CAMEL_INVALID_TYPE;
+	CamelLocalSummary *local_summary;
 
-	if (type == CAMEL_INVALID_TYPE) {
-		type = camel_type_register(camel_folder_summary_get_type(), "CamelLocalSummary",
-					   sizeof (CamelLocalSummary),
-					   sizeof (CamelLocalSummaryClass),
-					   (CamelObjectClassInitFunc) camel_local_summary_class_init,
-					   NULL,
-					   (CamelObjectInitFunc) camel_local_summary_init,
-					   (CamelObjectFinalizeFunc) camel_local_summary_finalise);
+	local_summary = CAMEL_LOCAL_SUMMARY (object);
+
+	if (local_summary->index != NULL) {
+		g_object_unref (local_summary->index);
+		local_summary->index = NULL;
 	}
 
-	return type;
+	/* Chain up to parent's dispose() method. */
+	G_OBJECT_CLASS (camel_local_summary_parent_class)->dispose (object);
 }
 
 static void
-camel_local_summary_class_init(CamelLocalSummaryClass *klass)
+local_summary_finalize (GObject *object)
 {
-	CamelFolderSummaryClass *sklass = (CamelFolderSummaryClass *) klass;
+	CamelLocalSummary *local_summary;
 
-	camel_local_summary_parent = CAMEL_FOLDER_SUMMARY_CLASS(camel_type_get_global_classfuncs(camel_folder_summary_get_type()));
+	local_summary = CAMEL_LOCAL_SUMMARY (object);
 
-	sklass->summary_header_load = summary_header_load;
-	sklass->summary_header_save = summary_header_save;
+	g_free (local_summary->folder_path);
 
-	sklass->summary_header_from_db = summary_header_from_db;
-	sklass->summary_header_to_db = summary_header_to_db;
-
-	sklass->message_info_new_from_header  = message_info_new_from_header;
-
-	klass->load = local_summary_load;
-	klass->check = local_summary_check;
-	klass->sync = local_summary_sync;
-	klass->add = local_summary_add;
-
-	klass->encode_x_evolution = local_summary_encode_x_evolution;
-	klass->decode_x_evolution = local_summary_decode_x_evolution;
-	klass->need_index = local_summary_need_index;
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (camel_local_summary_parent_class)->finalize (object);
 }
 
 static void
-camel_local_summary_init(CamelLocalSummary *obj)
+camel_local_summary_class_init (CamelLocalSummaryClass *class)
 {
-	struct _CamelFolderSummary *s = (CamelFolderSummary *)obj;
+	GObjectClass *object_class;
+	CamelFolderSummaryClass *folder_summary_class;
 
-	/* subclasses need to set the right instance data sizes */
-	s->message_info_size = sizeof(CamelLocalMessageInfo);
-	s->content_info_size = sizeof(CamelMessageContentInfo);
+	object_class = G_OBJECT_CLASS (class);
+	object_class->dispose = local_summary_dispose;
+	object_class->finalize = local_summary_finalize;
+
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (class);
+	folder_summary_class->message_info_size = sizeof (CamelLocalMessageInfo);
+	folder_summary_class->content_info_size = sizeof (CamelMessageContentInfo);
+	folder_summary_class->summary_header_load = summary_header_load;
+	folder_summary_class->summary_header_save = summary_header_save;
+	folder_summary_class->summary_header_from_db = summary_header_from_db;
+	folder_summary_class->summary_header_to_db = summary_header_to_db;
+	folder_summary_class->message_info_new_from_header  = message_info_new_from_header;
+
+	class->load = local_summary_load;
+	class->check = local_summary_check;
+	class->sync = local_summary_sync;
+	class->add = local_summary_add;
+	class->encode_x_evolution = local_summary_encode_x_evolution;
+	class->decode_x_evolution = local_summary_decode_x_evolution;
+	class->need_index = local_summary_need_index;
+}
+
+static void
+camel_local_summary_init (CamelLocalSummary *local_summary)
+{
+	CamelFolderSummary *folder_summary;
+
+	folder_summary = CAMEL_FOLDER_SUMMARY (local_summary);
 
 	/* and a unique file version */
-	s->version += CAMEL_LOCAL_SUMMARY_VERSION;
-}
-
-static void
-camel_local_summary_finalise(CamelObject *obj)
-{
-	CamelLocalSummary *mbs = CAMEL_LOCAL_SUMMARY(obj);
-
-	if (mbs->index)
-		camel_object_unref((CamelObject *)mbs->index);
-	g_free(mbs->folder_path);
+	folder_summary->version += CAMEL_LOCAL_SUMMARY_VERSION;
 }
 
 void
@@ -146,24 +138,28 @@ camel_local_summary_construct(CamelLocalSummary *new, const gchar *filename, con
 	new->folder_path = g_strdup(local_name);
 	new->index = index;
 	if (index)
-		camel_object_ref((CamelObject *)index);
+		g_object_ref (index);
 }
 
 static gint
-local_summary_load(CamelLocalSummary *cls, gint forceindex, CamelException *ex)
+local_summary_load(CamelLocalSummary *cls, gint forceindex, GError **error)
 {
 	d(g_print ("\nlocal_summary_load called \n"));
-	return camel_folder_summary_load_from_db ((CamelFolderSummary *)cls, ex);
+	return camel_folder_summary_load_from_db ((CamelFolderSummary *)cls, error);
 }
 
 /* load/check the summary */
 gint
-camel_local_summary_load(CamelLocalSummary *cls, gint forceindex, CamelException *ex)
+camel_local_summary_load(CamelLocalSummary *cls, gint forceindex, GError **error)
 {
+	CamelLocalSummaryClass *class;
+
 	d(printf("Loading summary ...\n"));
 
-	if ((forceindex && ((CamelLocalSummaryClass *)(CAMEL_OBJECT_GET_CLASS(cls)))->need_index())
-	    || ((CamelLocalSummaryClass *)(CAMEL_OBJECT_GET_CLASS(cls)))->load(cls, forceindex, ex) == -1) {
+	class = CAMEL_LOCAL_SUMMARY_GET_CLASS (cls);
+
+	if ((forceindex && class->need_index())
+	    || class->load(cls, forceindex, error) == -1) {
 		w(g_warning("Could not load summary: flags may be reset"));
 		camel_folder_summary_clear((CamelFolderSummary *)cls);
 		return -1;
@@ -180,13 +176,13 @@ void camel_local_summary_check_force(CamelLocalSummary *cls)
 gchar *
 camel_local_summary_encode_x_evolution(CamelLocalSummary *cls, const CamelLocalMessageInfo *info)
 {
-	return ((CamelLocalSummaryClass *)(CAMEL_OBJECT_GET_CLASS(cls)))->encode_x_evolution(cls, info);
+	return CAMEL_LOCAL_SUMMARY_GET_CLASS (cls)->encode_x_evolution (cls, info);
 }
 
 gint
 camel_local_summary_decode_x_evolution(CamelLocalSummary *cls, const gchar *xev, CamelLocalMessageInfo *info)
 {
-	return ((CamelLocalSummaryClass *)(CAMEL_OBJECT_GET_CLASS(cls)))->decode_x_evolution(cls, xev, info);
+	return CAMEL_LOCAL_SUMMARY_GET_CLASS (cls)->decode_x_evolution (cls, xev, info);
 }
 
 /*#define DOSTATS*/
@@ -270,11 +266,15 @@ do_stat_mi(CamelLocalSummary *cls, struct _stat_info *info, CamelMessageInfo *mi
 #endif
 
 gint
-camel_local_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, CamelException *ex)
+camel_local_summary_check (CamelLocalSummary *cls,
+                           CamelFolderChangeInfo *changeinfo,
+                           GError **error)
 {
+	CamelLocalSummaryClass *local_summary_class;
 	gint ret;
 
-	ret = ((CamelLocalSummaryClass *)(CAMEL_OBJECT_GET_CLASS(cls)))->check(cls, changeinfo, ex);
+	local_summary_class = CAMEL_LOCAL_SUMMARY_GET_CLASS (cls);
+	ret = local_summary_class->check(cls, changeinfo, error);
 
 #ifdef DOSTATS
 	if (ret != -1) {
@@ -301,15 +301,30 @@ camel_local_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changei
 }
 
 gint
-camel_local_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changeinfo, CamelException *ex)
+camel_local_summary_sync (CamelLocalSummary *cls,
+                          gboolean expunge,
+                          CamelFolderChangeInfo *changeinfo,
+                          GError **error)
 {
-	return ((CamelLocalSummaryClass *)(CAMEL_OBJECT_GET_CLASS(cls)))->sync(cls, expunge, changeinfo, ex);
+	CamelLocalSummaryClass *local_summary_class;
+
+	local_summary_class = CAMEL_LOCAL_SUMMARY_GET_CLASS (cls);
+
+	return local_summary_class->sync (cls, expunge, changeinfo, error);
 }
 
 CamelMessageInfo *
-camel_local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMessageInfo *info, CamelFolderChangeInfo *ci, CamelException *ex)
+camel_local_summary_add (CamelLocalSummary *cls,
+                         CamelMimeMessage *msg,
+                         const CamelMessageInfo *info,
+                         CamelFolderChangeInfo *ci,
+                         GError **error)
 {
-	return ((CamelLocalSummaryClass *)(CAMEL_OBJECT_GET_CLASS(cls)))->add(cls, msg, info, ci, ex);
+	CamelLocalSummaryClass *local_summary_class;
+
+	local_summary_class = CAMEL_LOCAL_SUMMARY_GET_CLASS (cls);
+
+	return local_summary_class->add (cls, msg, info, ci, error);
 }
 
 /**
@@ -401,27 +416,33 @@ camel_local_summary_write_headers(gint fd, struct _camel_header_raw *header, con
 }
 
 static gint
-local_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, CamelException *ex)
+local_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, GError **error)
 {
 	/* FIXME: sync index here ? */
 	return 0;
 }
 
 static gint
-local_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changeinfo, CamelException *ex)
+local_summary_sync (CamelLocalSummary *cls,
+                    gboolean expunge,
+                    CamelFolderChangeInfo *changeinfo,
+                    GError **error)
 {
-	gint ret = 0;
+	CamelFolderSummary *folder_summary;
 
-	ret = camel_folder_summary_save_to_db ((CamelFolderSummary *)cls, ex);
-	if (ret == -1) {
+	folder_summary = CAMEL_FOLDER_SUMMARY (cls);
+
+	if (camel_folder_summary_save_to_db (folder_summary, error) == -1) {
 		g_warning ("Could not save summary for local providers");
 		return -1;
 	}
 
-	if (cls->index && camel_index_sync(cls->index) == -1)
+	if (cls->index && camel_index_sync(cls->index) == -1) {
 		g_warning ("Could not sync index for %s: %s", cls->folder_path, g_strerror (errno));
+		return -1;
+	}
 
-	return ret;
+	return 0;
 }
 
 static gint
@@ -464,7 +485,7 @@ update_summary (CamelFolderSummary *summary, CamelMessageInfoBase *info, CamelMe
 }
 
 static CamelMessageInfo *
-local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMessageInfo *info, CamelFolderChangeInfo *ci, CamelException *ex)
+local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMessageInfo *info, CamelFolderChangeInfo *ci, GError **error)
 {
 	CamelLocalMessageInfo *mi;
 	CamelFolderSummary *s = (CamelFolderSummary *)cls;
@@ -499,9 +520,9 @@ local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMess
 		if (mi->info.size == 0) {
 			CamelStreamNull *sn = (CamelStreamNull *)camel_stream_null_new();
 
-			camel_data_wrapper_write_to_stream((CamelDataWrapper *)msg, (CamelStream *)sn);
+			camel_data_wrapper_write_to_stream((CamelDataWrapper *)msg, (CamelStream *)sn, NULL);
 			mi->info.size = sn->written;
-			camel_object_unref((CamelObject *)sn);
+			g_object_unref (sn);
 		}
 
 		mi->info.flags &= ~(CAMEL_MESSAGE_FOLDER_NOXEV|CAMEL_MESSAGE_FOLDER_FLAGGED);
@@ -511,8 +532,9 @@ local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMess
 		camel_folder_change_info_add_uid(ci, camel_message_info_uid(mi));
 	} else {
 		d(printf("Failed!\n"));
-		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
-				     _("Unable to add message to summary: unknown reason"));
+		g_set_error (
+			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+			_("Unable to add message to summary: unknown reason"));
 	}
 	return (CamelMessageInfo *)mi;
 }
@@ -645,7 +667,7 @@ summary_header_from_db (CamelFolderSummary *s, CamelFIRecord *fir)
 
 	/* We dont actually add our own headers, but version that we don't anyway */
 
-	if (((CamelFolderSummaryClass *)camel_local_summary_parent)->summary_header_from_db(s, fir) == -1)
+	if (CAMEL_FOLDER_SUMMARY_CLASS (camel_local_summary_parent_class)->summary_header_from_db(s, fir) == -1)
 		return -1;
 
 	part = fir->bdata;
@@ -668,7 +690,7 @@ summary_header_load(CamelFolderSummary *s, FILE *in)
 
 	/* We dont actually add our own headers, but version that we don't anyway */
 
-	if (((CamelFolderSummaryClass *)camel_local_summary_parent)->summary_header_load(s, in) == -1)
+	if (CAMEL_FOLDER_SUMMARY_CLASS (camel_local_summary_parent_class)->summary_header_load(s, in) == -1)
 		return -1;
 
 	/* Legacy version, version is in summary only */
@@ -680,11 +702,14 @@ summary_header_load(CamelFolderSummary *s, FILE *in)
 }
 
 static struct _CamelFIRecord *
-summary_header_to_db (CamelFolderSummary *s, CamelException *ex)
+summary_header_to_db (CamelFolderSummary *s, GError **error)
 {
+	CamelFolderSummaryClass *folder_summary_class;
 	struct _CamelFIRecord *fir;
 
-	fir = ((CamelFolderSummaryClass *)camel_local_summary_parent)->summary_header_to_db (s, ex);
+	/* Chain up to parent's summary_header_to_db() method. */
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (camel_local_summary_parent_class);
+	fir = folder_summary_class->summary_header_to_db (s, NULL);
 	if (fir)
 		fir->bdata = g_strdup_printf ("%d", CAMEL_LOCAL_SUMMARY_VERSION);
 
@@ -696,7 +721,7 @@ summary_header_save(CamelFolderSummary *s, FILE *out)
 {
 	/*CamelLocalSummary *cls = (CamelLocalSummary *)s;*/
 
-	if (((CamelFolderSummaryClass *)camel_local_summary_parent)->summary_header_save(s, out) == -1)
+	if (CAMEL_FOLDER_SUMMARY_CLASS (camel_local_summary_parent_class)->summary_header_save(s, out) == -1)
 		return -1;
 
 	return camel_file_util_encode_fixed_int32(out, CAMEL_LOCAL_SUMMARY_VERSION);
@@ -708,7 +733,7 @@ message_info_new_from_header(CamelFolderSummary *s, struct _camel_header_raw *h)
 	CamelLocalMessageInfo *mi;
 	CamelLocalSummary *cls = (CamelLocalSummary *)s;
 
-	mi = (CamelLocalMessageInfo *)((CamelFolderSummaryClass *)camel_local_summary_parent)->message_info_new_from_header(s, h);
+	mi = (CamelLocalMessageInfo *)CAMEL_FOLDER_SUMMARY_CLASS (camel_local_summary_parent_class)->message_info_new_from_header(s, h);
 	if (mi) {
 		const gchar *xev;
 		gint doindex = FALSE;
