@@ -30,6 +30,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 
 #include "camel-maildir-folder.h"
@@ -38,12 +39,12 @@
 
 #define d(x)
 
-static CamelFolder *get_folder(CamelStore * store, const gchar *folder_name, guint32 flags, CamelException * ex);
-static CamelFolder *get_inbox (CamelStore *store, CamelException *ex);
-static gboolean delete_folder(CamelStore * store, const gchar *folder_name, CamelException * ex);
-static gboolean maildir_rename_folder(CamelStore *store, const gchar *old, const gchar *new, CamelException *ex);
+static CamelFolder *get_folder(CamelStore * store, const gchar *folder_name, guint32 flags, GError **error);
+static CamelFolder *get_inbox (CamelStore *store, GError **error);
+static gboolean delete_folder(CamelStore * store, const gchar *folder_name, GError **error);
+static gboolean maildir_rename_folder(CamelStore *store, const gchar *old, const gchar *new, GError **error);
 
-static CamelFolderInfo * get_folder_info (CamelStore *store, const gchar *top, guint32 flags, CamelException *ex);
+static CamelFolderInfo * get_folder_info (CamelStore *store, const gchar *top, guint32 flags, GError **error);
 
 static gboolean maildir_compare_folder_name(gconstpointer a, gconstpointer b);
 static guint maildir_hash_folder_name(gconstpointer a);
@@ -102,7 +103,7 @@ static CamelFolder *
 get_folder (CamelStore *store,
             const gchar *folder_name,
             guint32 flags,
-            CamelException * ex)
+            GError **error)
 {
 	CamelStoreClass *store_class;
 	gchar *name, *tmp, *cur, *new;
@@ -113,7 +114,7 @@ get_folder (CamelStore *store,
 
 	/* Chain up to parent's get_folder() method. */
 	store_class = CAMEL_STORE_CLASS (camel_maildir_store_parent_class);
-	if (!store_class->get_folder (store, folder_name, flags, ex))
+	if (!store_class->get_folder (store, folder_name, flags, error))
 		return NULL;
 
 	name = g_strdup_printf("%s%s", CAMEL_LOCAL_STORE(store)->toplevel_dir, folder_name);
@@ -123,14 +124,15 @@ get_folder (CamelStore *store,
 
 	if (!strcmp(folder_name, ".")) {
 		/* special case "." (aka inbox), may need to be created */
-		if (stat(tmp, &st) != 0 || !S_ISDIR(st.st_mode)
-		    || stat(cur, &st) != 0 || !S_ISDIR(st.st_mode)
-		    || stat(new, &st) != 0 || !S_ISDIR(st.st_mode)) {
+		if (g_stat(tmp, &st) != 0 || !S_ISDIR(st.st_mode)
+		    || g_stat(cur, &st) != 0 || !S_ISDIR(st.st_mode)
+		    || g_stat(new, &st) != 0 || !S_ISDIR(st.st_mode)) {
 			if (mkdir(tmp, 0700) != 0
 			    || mkdir(cur, 0700) != 0
 			    || mkdir(new, 0700) != 0) {
-				camel_exception_setv (
-					ex, CAMEL_EXCEPTION_SYSTEM,
+				g_set_error (
+					error, G_IO_ERROR,
+					g_io_error_from_errno (errno),
 					_("Cannot create folder '%s': %s"),
 					folder_name, g_strerror(errno));
 				rmdir(tmp);
@@ -139,17 +141,19 @@ get_folder (CamelStore *store,
 				goto fail;
 			}
 		}
-		folder = camel_maildir_folder_new(store, folder_name, flags, ex);
-	} else if (stat(name, &st) == -1) {
+		folder = camel_maildir_folder_new(store, folder_name, flags, error);
+	} else if (g_stat(name, &st) == -1) {
 		/* folder doesn't exist, see if we should create it */
 		if (errno != ENOENT) {
-			camel_exception_setv (
-				ex, CAMEL_EXCEPTION_SYSTEM,
+			g_set_error (
+				error, G_IO_ERROR,
+				g_io_error_from_errno (errno),
 				_("Cannot get folder '%s': %s"),
 				folder_name, g_strerror (errno));
 		} else if ((flags & CAMEL_STORE_FOLDER_CREATE) == 0) {
-			camel_exception_setv (
-				ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+			g_set_error (
+				error, CAMEL_STORE_ERROR,
+				CAMEL_STORE_ERROR_NO_FOLDER,
 				_("Cannot get folder '%s': folder does not exist."),
 				folder_name);
 		} else {
@@ -157,8 +161,9 @@ get_folder (CamelStore *store,
 			    || mkdir(tmp, 0700) != 0
 			    || mkdir(cur, 0700) != 0
 			    || mkdir(new, 0700) != 0) {
-				camel_exception_setv (
-					ex, CAMEL_EXCEPTION_SYSTEM,
+				g_set_error (
+					error, G_IO_ERROR,
+					g_io_error_from_errno (errno),
 					_("Cannot create folder '%s': %s"),
 					folder_name, g_strerror (errno));
 				rmdir(tmp);
@@ -166,25 +171,25 @@ get_folder (CamelStore *store,
 				rmdir(new);
 				rmdir(name);
 			} else {
-				folder = camel_maildir_folder_new(store, folder_name, flags, ex);
+				folder = camel_maildir_folder_new(store, folder_name, flags, error);
 			}
 		}
 	} else if (!S_ISDIR(st.st_mode)
-		   || stat(tmp, &st) != 0 || !S_ISDIR(st.st_mode)
-		   || stat(cur, &st) != 0 || !S_ISDIR(st.st_mode)
-		   || stat(new, &st) != 0 || !S_ISDIR(st.st_mode)) {
+		   || g_stat(tmp, &st) != 0 || !S_ISDIR(st.st_mode)
+		   || g_stat(cur, &st) != 0 || !S_ISDIR(st.st_mode)
+		   || g_stat(new, &st) != 0 || !S_ISDIR(st.st_mode)) {
 		/* folder exists, but not maildir */
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_SYSTEM,
+		g_set_error (
+			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot get folder '%s': not a maildir directory."),
 			name);
 	} else if (flags & CAMEL_STORE_FOLDER_EXCL) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_SYSTEM,
+		g_set_error (
+			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot create folder '%s': folder exists."),
 			folder_name);
 	} else {
-		folder = camel_maildir_folder_new(store, folder_name, flags, ex);
+		folder = camel_maildir_folder_new(store, folder_name, flags, error);
 	}
 fail:
 	g_free(name);
@@ -197,24 +202,25 @@ fail:
 
 static CamelFolder *
 get_inbox (CamelStore *store,
-           CamelException *ex)
+           GError **error)
 {
 	return camel_store_get_folder (
-		store, ".", CAMEL_STORE_FOLDER_CREATE, ex);
+		store, ".", CAMEL_STORE_FOLDER_CREATE, error);
 }
 
 static gboolean
 delete_folder (CamelStore *store,
                const gchar *folder_name,
-               CamelException *ex)
+               GError **error)
 {
 	gchar *name, *tmp, *cur, *new;
 	struct stat st;
 	gboolean success = TRUE;
 
 	if (strcmp(folder_name, ".") == 0) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+		g_set_error (
+			error, CAMEL_STORE_ERROR,
+			CAMEL_STORE_ERROR_NO_FOLDER,
 			_("Cannot delete folder: %s: Invalid operation"),
 			_("Inbox"));
 		return FALSE;
@@ -226,12 +232,13 @@ delete_folder (CamelStore *store,
 	cur = g_strdup_printf("%s/cur", name);
 	new = g_strdup_printf("%s/new", name);
 
-	if (stat(name, &st) == -1 || !S_ISDIR(st.st_mode)
-	    || stat(tmp, &st) == -1 || !S_ISDIR(st.st_mode)
-	    || stat(cur, &st) == -1 || !S_ISDIR(st.st_mode)
-	    || stat(new, &st) == -1 || !S_ISDIR(st.st_mode)) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_SYSTEM,
+	if (g_stat(name, &st) == -1 || !S_ISDIR(st.st_mode)
+	    || g_stat(tmp, &st) == -1 || !S_ISDIR(st.st_mode)
+	    || g_stat(cur, &st) == -1 || !S_ISDIR(st.st_mode)
+	    || g_stat(new, &st) == -1 || !S_ISDIR(st.st_mode)) {
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
 			_("Could not delete folder '%s': %s"),
 			folder_name, errno ? g_strerror (errno) :
 			_("not a maildir directory"));
@@ -269,8 +276,9 @@ delete_folder (CamelStore *store,
 			mkdir(cur, 0700);
 			mkdir(new, 0700);
 			mkdir(tmp, 0700);
-			camel_exception_setv (
-				ex, CAMEL_EXCEPTION_SYSTEM,
+			g_set_error (
+				error, G_IO_ERROR,
+				g_io_error_from_errno (err),
 				_("Could not delete folder '%s': %s"),
 				folder_name, g_strerror (err));
 		} else {
@@ -279,7 +287,7 @@ delete_folder (CamelStore *store,
 			/* Chain up to parent's delete_folder() method. */
 			store_class = CAMEL_STORE_CLASS (camel_maildir_store_parent_class);
 			success = store_class->delete_folder (
-				store, folder_name, ex);
+				store, folder_name, error);
 		}
 	}
 
@@ -295,13 +303,14 @@ static gboolean
 maildir_rename_folder (CamelStore *store,
                        const gchar *old,
                        const gchar *new,
-                       CamelException *ex)
+                       GError **error)
 {
 	CamelStoreClass *store_class;
 
 	if (strcmp(old, ".") == 0) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+		g_set_error (
+			error, CAMEL_STORE_ERROR,
+			CAMEL_STORE_ERROR_NO_FOLDER,
 			_("Cannot rename folder: %s: Invalid operation"),
 			_("Inbox"));
 		return FALSE;
@@ -309,7 +318,7 @@ maildir_rename_folder (CamelStore *store,
 
 	/* Chain up to parent's rename_folder() method. */
 	store_class = CAMEL_STORE_CLASS (camel_maildir_store_parent_class);
-	return store_class->rename_folder(store, old, new, ex);
+	return store_class->rename_folder(store, old, new, error);
 }
 
 static void
@@ -319,7 +328,7 @@ fill_fi (CamelStore *store,
 {
 	CamelFolder *folder;
 
-	folder = camel_object_bag_get(store->folders, fi->full_name);
+	folder = camel_object_bag_peek (store->folders, fi->full_name);
 
 	if (folder == NULL
 	    && (flags & CAMEL_STORE_FOLDER_INFO_FAST) == 0)
@@ -349,6 +358,11 @@ fill_fi (CamelStore *store,
 		g_free(folderpath);
 		g_free(path);
 	}
+
+	if (camel_local_store_is_main_store (CAMEL_LOCAL_STORE (store)) && fi->full_name
+	    && (fi->flags & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_NORMAL)
+		fi->flags = (fi->flags & ~CAMEL_FOLDER_TYPE_MASK)
+			    | camel_local_store_get_folder_type_by_full_name (CAMEL_LOCAL_STORE (store), fi->full_name);
 }
 
 struct _scan_node {
@@ -413,9 +427,9 @@ scan_fi (CamelStore *store,
 	cur = g_build_filename(url->path, fi->full_name, "cur", NULL);
 	new = g_build_filename(url->path, fi->full_name, "new", NULL);
 
-	if (!(stat(tmp, &st) == 0 && S_ISDIR(st.st_mode)
-	      && stat(cur, &st) == 0 && S_ISDIR(st.st_mode)
-	      && stat(new, &st) == 0 && S_ISDIR(st.st_mode)))
+	if (!(g_stat(tmp, &st) == 0 && S_ISDIR(st.st_mode)
+	      && g_stat(cur, &st) == 0 && S_ISDIR(st.st_mode)
+	      && g_stat(new, &st) == 0 && S_ISDIR(st.st_mode)))
 		fi->flags |= CAMEL_FOLDER_NOSELECT;
 
 	g_free(new);
@@ -432,7 +446,7 @@ scan_dirs (CamelStore *store,
            guint32 flags,
            CamelFolderInfo *topfi,
            CamelURL *url,
-           CamelException *ex)
+           GError **error)
 {
 	CamelDList queue = CAMEL_DLIST_INITIALISER(queue);
 	struct _scan_node *sn;
@@ -467,8 +481,9 @@ scan_dirs (CamelStore *store,
 		dir = opendir(name);
 		if (dir == NULL) {
 			g_free(name);
-			camel_exception_setv (
-				ex, CAMEL_EXCEPTION_SYSTEM,
+			g_set_error (
+				error, G_IO_ERROR,
+				g_io_error_from_errno (errno),
 				_("Could not scan folder '%s': %s"),
 				root, g_strerror (errno));
 			goto fail;
@@ -484,7 +499,7 @@ scan_dirs (CamelStore *store,
 				continue;
 
 			tmp = g_build_filename(name, d->d_name, NULL);
-			if (stat(tmp, &st) == 0 && S_ISDIR(st.st_mode)) {
+			if (g_stat(tmp, &st) == 0 && S_ISDIR(st.st_mode)) {
 				struct _scan_node in;
 
 				in.dnode = st.st_dev;
@@ -536,7 +551,7 @@ static CamelFolderInfo *
 get_folder_info (CamelStore *store,
                  const gchar *top,
                  guint32 flags,
-                 CamelException *ex)
+                 GError **error)
 {
 	CamelFolderInfo *fi = NULL;
 	CamelLocalStore *local_store = (CamelLocalStore *)store;
@@ -550,7 +565,7 @@ get_folder_info (CamelStore *store,
 
 		/* create a dummy "." parent inbox, use to scan, then put back at the top level */
 		fi = scan_fi(store, flags, url, ".", _("Inbox"));
-		if (scan_dirs(store, flags, fi, url, ex) == -1)
+		if (scan_dirs(store, flags, fi, url, error) == -1)
 			goto fail;
 		fi->next = fi->child;
 		scan = fi->child;
@@ -568,7 +583,7 @@ get_folder_info (CamelStore *store,
 		const gchar *name = strrchr(top, '/');
 
 		fi = scan_fi(store, flags, url, top, name?name+1:top);
-		if (scan_dirs(store, flags, fi, url, ex) == -1)
+		if (scan_dirs(store, flags, fi, url, error) == -1)
 			goto fail;
 	}
 
