@@ -24,44 +24,59 @@
  */
 
 #ifndef CAMEL_POP3_STORE_H
-#define CAMEL_POP3_STORE_H 1
+#define CAMEL_POP3_STORE_H
+
+#include <camel/camel.h>
 
-#include <camel/camel-types.h>
-#include <camel/camel-store.h>
 #include "camel-pop3-engine.h"
 
-#define CAMEL_POP3_STORE_TYPE     (camel_pop3_store_get_type ())
-#define CAMEL_POP3_STORE(obj)     (CAMEL_CHECK_CAST((obj), CAMEL_POP3_STORE_TYPE, CamelPOP3Store))
-#define CAMEL_POP3_STORE_CLASS(k) (CAMEL_CHECK_CLASS_CAST ((k), CAMEL_POP3_STORE_TYPE, CamelPOP3StoreClass))
-#define CAMEL_IS_POP3_STORE(o)    (CAMEL_CHECK_TYPE((o), CAMEL_POP3_STORE_TYPE))
+/* Standard GObject macros */
+#define CAMEL_TYPE_POP3_STORE \
+	(camel_pop3_store_get_type ())
+#define CAMEL_POP3_STORE(obj) \
+	(G_TYPE_CHECK_INSTANCE_CAST \
+	((obj), CAMEL_TYPE_POP3_STORE, CamelPOP3Store))
+#define CAMEL_POP3_STORE_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_CAST \
+	((cls), CAMEL_TYPE_POP3_STORE, CamelPOP3StoreClass))
+#define CAMEL_IS_POP3_STORE(obj) \
+	(G_TYPE_CHECK_INSTANCE_TYPE \
+	((obj), CAMEL_TYPE_POP3_STORE))
+#define CAMEL_IS_POP3_STORE_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_TYPE \
+	((cls), CAMEL_TYPE_POP3_STORE))
+#define CAMEL_POP3_STORE_GET_CLASS(obj) \
+	(G_TYPE_INSTANCE_GET_CLASS \
+	((obj), CAMEL_TYPE_POP3_STORE, CamelPOP3StoreClass))
 
 G_BEGIN_DECLS
 
-typedef struct {
-	CamelStore parent_object;
+typedef struct _CamelPOP3Store CamelPOP3Store;
+typedef struct _CamelPOP3StoreClass CamelPOP3StoreClass;
+
+struct _CamelPOP3Store {
+	CamelStore parent;
 
 	CamelPOP3Engine *engine; /* pop processing engine */
 
-	struct _CamelDataCache *cache;
+	CamelDataCache *cache;
 
 	guint delete_after;
-} CamelPOP3Store;
+};
 
-typedef struct {
+struct _CamelPOP3StoreClass {
 	CamelStoreClass parent_class;
+};
 
-} CamelPOP3StoreClass;
+GType camel_pop3_store_get_type (void);
 
 /* public methods */
-void camel_pop3_store_expunge (CamelPOP3Store *store, CamelException *ex);
+void camel_pop3_store_expunge (CamelPOP3Store *store, GError **error);
 
 /* support functions */
 enum { CAMEL_POP3_OK, CAMEL_POP3_ERR, CAMEL_POP3_FAIL };
-gint camel_pop3_command (CamelPOP3Store *store, gchar **ret, CamelException *ex, gchar *fmt, ...);
-gchar *camel_pop3_command_get_additional_data (CamelPOP3Store *store, gint total, CamelException *ex);
-
-/* Standard Camel function */
-CamelType camel_pop3_store_get_type (void);
+gint camel_pop3_command (CamelPOP3Store *store, gchar **ret, GError **error, gchar *fmt, ...);
+gchar *camel_pop3_command_get_additional_data (CamelPOP3Store *store, gint total, GError **error);
 
 G_END_DECLS
 
