@@ -44,6 +44,8 @@
  * modified or freed.
  *
  * Returns: base directory for user-specific, non-essential data
+ *
+ * Since: 2.32
  **/
 const gchar *
 e_get_user_cache_dir (void)
@@ -51,8 +53,8 @@ e_get_user_cache_dir (void)
 	static gchar *dirname = NULL;
 
 	if (G_UNLIKELY (dirname == NULL)) {
-		const gchar *data_dir = e_get_user_data_dir ();
-		dirname = g_build_filename (data_dir, "cache", NULL);
+		const gchar *cache_dir = g_get_user_cache_dir ();
+		dirname = g_build_filename (cache_dir, "evolution", NULL);
 		g_mkdir_with_parents (dirname, 0700);
 	}
 
@@ -69,11 +71,21 @@ e_get_user_cache_dir (void)
  * modified or freed.
  *
  * Returns: base directory for user-specific configuration information
+ *
+ * Since: 2.32
  **/
 const gchar *
 e_get_user_config_dir (void)
 {
-	return e_get_user_data_dir ();
+	static gchar *dirname = NULL;
+
+	if (G_UNLIKELY (dirname == NULL)) {
+		const gchar *config_dir = g_get_user_config_dir ();
+		dirname = g_build_filename (config_dir, "evolution", NULL);
+		g_mkdir_with_parents (dirname, 0700);
+	}
+
+	return dirname;
 }
 
 /**
@@ -86,6 +98,8 @@ e_get_user_config_dir (void)
  * modified or freed.
  *
  * Returns: base directory for user-specific data
+ *
+ * Since: 2.32
  **/
 const gchar *
 e_get_user_data_dir (void)
@@ -93,8 +107,8 @@ e_get_user_data_dir (void)
 	static gchar *dirname = NULL;
 
 	if (G_UNLIKELY (dirname == NULL)) {
-		const gchar *home_dir = g_get_home_dir ();
-		dirname = g_build_filename (home_dir, ".evolution", NULL);
+		const gchar *data_dir = g_get_user_data_dir ();
+		dirname = g_build_filename (data_dir, "evolution", NULL);
 		g_mkdir_with_parents (dirname, 0700);
 	}
 
@@ -383,16 +397,16 @@ e_util_utf8_remove_accents (const gchar *str)
 	if (!res)
 		return g_strdup (str);
 
-	for (i = 0, j = 0; res [i]; i++) {
-		if ((guchar)res[i] != 0xCC || res [i + 1] == 0) {
-			res [j] = res [i];
+	for (i = 0, j = 0; res[i]; i++) {
+		if ((guchar)res[i] != 0xCC || res[i + 1] == 0) {
+			res[j] = res[i];
 			j++;
 		} else {
 			i++;
 		}
 	}
 
-	res [j] = 0;
+	res[j] = 0;
 
 	return res;
 }
@@ -535,6 +549,8 @@ e_utf8_strftime (gchar *string,
  * reused after a thread has terminated, though.
  *
  * Returns: A 64-bit integer.
+ *
+ * Since: 2.32
  */
 guint64
 e_util_gthread_id (GThread *thread)
