@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
- * Copyright (C) 2005 Novell, Inc.
+/*
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __E_XML_UTILS_H__
@@ -26,8 +26,8 @@
 
 G_BEGIN_DECLS
 
-xmlDocPtr   e_xml_parse_file (const char *filename);
-int         e_xml_save_file  (const char *filename,
+xmlDocPtr   e_xml_parse_file (const gchar *filename);
+gint         e_xml_save_file  (const gchar *filename,
 			      xmlDocPtr   doc);
 xmlNode    *e_xml_get_child_by_name (const xmlNode *parent,
 				     const xmlChar *child_name);
