@@ -39,13 +39,13 @@ void e_passwords_init (void);
 
 void        e_passwords_shutdown          (void);
 void	    e_passwords_cancel(void);
-void        e_passwords_set_online(int state);
-void        e_passwords_remember_password (const char *component, const char *key);
-void        e_passwords_add_password      (const char *key, const char *passwd);
-char       *e_passwords_get_password      (const char *component, const char *key);
-void        e_passwords_forget_password   (const char *component, const char *key);
+void        e_passwords_set_online(gint state);
+void        e_passwords_remember_password (const gchar *component, const gchar *key);
+void        e_passwords_add_password      (const gchar *key, const gchar *passwd);
+gchar       *e_passwords_get_password      (const gchar *component, const gchar *key);
+void        e_passwords_forget_password   (const gchar *component, const gchar *key);
 void        e_passwords_forget_passwords  (void);
-void        e_passwords_clear_passwords (const char *component);
+void        e_passwords_clear_passwords (const gchar *component);
 
 typedef enum {
 	E_PASSWORDS_REMEMBER_NEVER,
@@ -61,9 +61,9 @@ typedef enum {
 	E_PASSWORDS_PASSPHRASE = 1<<12 /* We are asking a passphrase */
 } EPasswordsRememberType;
 
-char *      e_passwords_ask_password      (const char *title,
-					   const char*component_name, const char *key,
-					   const char *prompt,
+gchar *      e_passwords_ask_password      (const gchar *title,
+					   const gchar *component_name, const gchar *key,
+					   const gchar *prompt,
 					   EPasswordsRememberType remember_type,
 					   gboolean *remember,
 					   GtkWindow *parent);
