@@ -4,7 +4,7 @@
  *  JP Rosevear <jpr@ximian.com>
  *  Rodrigo Moya <rodrigo@ximian.com>
  *
- * Copyright 2003, Novell, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -26,51 +26,8 @@
 #include <libsoup/soup-uri.h>
 #include "e-gw-message.h"
 
-static void
-print_header (gpointer name, gpointer value, gpointer data)
-{
-	g_print ("%s: %s\n", (char *) name, (char *) value);
-}
-
-static void
-debug_handler (SoupMessage *msg, gpointer user_data)
-{
-	g_print ("%d %s\nSOAP-Debug: %p @ %lu\n",
-                msg->status_code, msg->reason_phrase,
-                msg, time (NULL));
-
-	/* print headers */
-	soup_message_foreach_header (msg->response_headers, print_header, NULL);
-
-	/* print response */
-	if (msg->response.length) {
-		fputc ('\n', stdout);
-		fwrite (msg->response.body, 1, msg->response.length, stdout);
-		fputc ('\n', stdout);
-	}
-}
-
-static void
-setup_debug (SoupSoapMessage *msg)
-{
-	const SoupUri *suri;
-
-	suri = soup_message_get_uri (SOUP_MESSAGE (msg));
-	g_print ("%s %s%s%s HTTP/1.1\nSOAP-Debug: %p @ %lu\n",
-		 SOUP_MESSAGE (msg)->method, suri->path,
-		 suri->query ? "?" : "",
-		 suri->query ? suri->query : "",
-		 msg, (unsigned long) time (NULL));
-
-	/* print message headers */
-	print_header ("Host", suri->host, NULL);
-	soup_message_foreach_header (SOUP_MESSAGE (msg)->request_headers, print_header, NULL);
-
-	soup_message_add_handler (SOUP_MESSAGE (msg), SOUP_HANDLER_POST_BODY, debug_handler, NULL);
-}
-
 SoupSoapMessage *
-e_gw_message_new_with_header (const char *uri, const char *session_id, const char *method_name)
+e_gw_message_new_with_header (const gchar *uri, const gchar *session_id, const gchar *method_name)
 {
 	SoupSoapMessage *msg;
 
@@ -80,16 +37,11 @@ e_gw_message_new_with_header (const char *uri, const char *session_id, const cha
 		return NULL;
 	}
 
-	soup_message_add_header (SOUP_MESSAGE (msg)->request_headers, "Content-Type", "text/xml");
-	soup_message_add_header (SOUP_MESSAGE (msg)->request_headers, "User-Agent",
-				 "Evolution/" VERSION);
-	soup_message_add_header (SOUP_MESSAGE (msg)->request_headers,"Connection",  "Keep-Alive");
-	soup_message_add_header (SOUP_MESSAGE (msg)->request_headers, "SOAPAction", method_name);
-
-	if (g_getenv ("GROUPWISE_DEBUG")) {
-		if (atoi (g_getenv ("GROUPWISE_DEBUG")) == 1)
-			setup_debug (msg);
-	}
+	soup_message_headers_append (SOUP_MESSAGE (msg)->request_headers, "Content-Type", "text/xml");
+	soup_message_headers_append (SOUP_MESSAGE (msg)->request_headers, "User-Agent",
+				     "Evolution/" VERSION);
+	soup_message_headers_append (SOUP_MESSAGE (msg)->request_headers,"Connection",  "Keep-Alive");
+	soup_message_headers_append (SOUP_MESSAGE (msg)->request_headers, "SOAPAction", method_name);
 
 	soup_soap_message_start_envelope (msg);
 	if (session_id && *session_id) {
@@ -111,7 +63,7 @@ e_gw_message_new_with_header (const char *uri, const char *session_id, const cha
 }
 
 void
-e_gw_message_write_string_parameter (SoupSoapMessage *msg, const char *name, const char *prefix, const char *value)
+e_gw_message_write_string_parameter (SoupSoapMessage *msg, const gchar *name, const gchar *prefix, const gchar *value)
 {
 	soup_soap_message_start_element (msg, name, prefix, NULL);
 	soup_soap_message_write_string (msg, value);
@@ -120,11 +72,11 @@ e_gw_message_write_string_parameter (SoupSoapMessage *msg, const char *name, con
 
 void
 e_gw_message_write_string_parameter_with_attribute (SoupSoapMessage *msg,
-						    const char *name,
-						    const char *prefix,
-						    const char *value,
-						    const char *attribute_name,
-						    const char *attribute_value)
+						    const gchar *name,
+						    const gchar *prefix,
+						    const gchar *value,
+						    const gchar *attribute_name,
+						    const gchar *attribute_value)
 {
 	soup_soap_message_start_element (msg, name, prefix, NULL);
 	soup_soap_message_add_attribute (msg, attribute_name, attribute_value, NULL, NULL);
@@ -133,7 +85,7 @@ e_gw_message_write_string_parameter_with_attribute (SoupSoapMessage *msg,
 }
 
 void
-e_gw_message_write_base64_parameter (SoupSoapMessage *msg, const char *name, const char *prefix, const char *value)
+e_gw_message_write_base64_parameter (SoupSoapMessage *msg, const gchar *name, const gchar *prefix, const gchar *value)
 {
 	soup_soap_message_start_element (msg, name, prefix, NULL);
 	soup_soap_message_write_base64 (msg, value, strlen (value));
@@ -141,7 +93,7 @@ e_gw_message_write_base64_parameter (SoupSoapMessage *msg, const char *name, con
 }
 
 void
-e_gw_message_write_int_parameter (SoupSoapMessage *msg, const char *name, const char *prefix, long value)
+e_gw_message_write_int_parameter (SoupSoapMessage *msg, const gchar *name, const gchar *prefix, glong value)
 {
 	soup_soap_message_start_element (msg, name, prefix, NULL);
 	soup_soap_message_write_int (msg, value);
@@ -158,13 +110,15 @@ e_gw_message_write_footer (SoupSoapMessage *msg)
 	soup_soap_message_persist (msg);
 
 	if (g_getenv ("GROUPWISE_DEBUG") && (atoi (g_getenv ("GROUPWISE_DEBUG")) == 1)) {
-		const char *header = soup_message_get_header (SOUP_MESSAGE (msg)->request_headers, "SOAPAction");
+		const gchar *header = soup_message_headers_get (SOUP_MESSAGE (msg)->request_headers, "SOAPAction");
+
+		soup_buffer_free (soup_message_body_flatten (SOUP_MESSAGE (msg)->request_body));
 		if (header && g_str_equal (header, "loginRequest")) {
 			gchar *body;
 			gchar *begin = NULL;
 			gchar *end = NULL;
 
-			body = g_strndup (SOUP_MESSAGE (msg)->request.body, SOUP_MESSAGE (msg)->request.length);
+			body = g_strdup (SOUP_MESSAGE (msg)->request_body->data);
 			begin = g_strrstr (body, "<types:password>");
 			if (begin)
 				begin = begin + strlen ("<types:password>");
@@ -176,7 +130,7 @@ e_gw_message_write_footer (SoupSoapMessage *msg)
 
 			}
 			fputc ('\n', stdout);
-			fwrite (body, 1, SOUP_MESSAGE (msg)->request.length, stdout);
+			fputs (body, stdout);
 			fputc ('\n', stdout);
 			g_free (body);
 		}
@@ -184,7 +138,7 @@ e_gw_message_write_footer (SoupSoapMessage *msg)
 
 			/* print request's body */
 			fputc ('\n', stdout);
-			fwrite (SOUP_MESSAGE (msg)->request.body, 1, SOUP_MESSAGE (msg)->request.length, stdout);
+			fputs (SOUP_MESSAGE (msg)->request_body->data, stdout);
 			fputc ('\n', stdout);
 		}
 	}
